//! Generate register interface software from register map in XML.
//!
//! # Example
//!
//! Here's a typical usage:
//! ```no_run
//! use endcap_sl_software_ri_generator::types;
//!
//! let xmlfile = std::fs::read_to_string("./csr.xml")?;
//! let doc = roxmltree::Document::parse_with_options(
//!     &xmlfile,
//!     roxmltree::ParsingOptions {
//!         allow_dtd: true,
//!         nodes_limit: u32::MAX,
//!     },
//! )?;
//!
//! let register_map = types::Module::from_xml_dom(doc.root_element())?;
//!
//! let files = register_map.generate_code()?;
//! endcap_sl_software_ri_generator::write_to_files(files, &std::path::PathBuf::from("out"))?;
//!
//! # Ok::<(), anyhow::Error>(())
//! ```
//!
//! # Overview
//!
//! 1. Convert [`roxmltree::Document`] to register map respresented with types defined in
//!    [`types`], filling missing parameters. See [`converter`].
//! 2. Generate [`proc_macro2::TokenStream`] from register map produced in the previous step. See
//!    [`generator`].
//!
//! # modules
//! - [`types`]: type definitions of internal register map representation
//! - [`converter`]: DOM to internal representation
//! - [`generator`]: internal representation to rust code
//! - [`io`]: formatting and printing

pub mod converter;
pub mod generator;
pub mod io;
pub mod meta;
mod parser;
mod type_traits;
pub mod types;

pub use io::write_to_files;
